@@ -1,8 +1,8 @@
 #!/usr/bin/perl
-# DocumentId:	$Id: debarchiver.pl,v 1.25 2002/01/26 22:59:04 ola Exp $
+# DocumentId:	$Id: debarchiver.pl,v 1.40 2003/08/28 09:45:27 ola Exp $
 # Author:	$Author: ola $
-# Date:		$Date: 2002/01/26 22:59:04 $
-# Version:	$Revision: 1.25 $
+# Date:		$Date: 2003/08/28 09:45:27 $
+# Version:	$Revision: 1.40 $
 # Summary:
 #	This program reads a config file, traverse through all .changes files
 #	in the specified directory and puts the files into the right place.
@@ -26,13 +26,16 @@ use OpaL::read qw(readfile readcommand);
 
 $copycmd = "cp -af";
 $rmcmd = "rm -f";
+$mailcmd = "mail";
 $movecmd = "mv";
+$vrfycmd = "dscverify";
 $inputdir = "/var/lib/debarchiver/incoming";
 $destdir = "/var/lib/debarchiver/dists";
 $cinstall = "installed";
 $lockfile = "debarchiver.lock";
 $etcconfigfile = "/etc/debarchiver.conf";
 $inputconfigfile = "input.conf";
+$verifysignatures = 0;
 $userconfigfile = "$ENV{HOME}/.debarchiver.conf";
 
 ###############################################################################
@@ -41,6 +44,8 @@ $userconfigfile = "$ENV{HOME}/.debarchiver.conf";
 
 # Indexed by config name.
 %CConf = ();
+# Indexed by meta type.
+%CMeta = ();
 # Indexed by file name.
 %CFiles = ();
 # Indexed by package name.
@@ -61,8 +66,8 @@ $distinputcriteria = "^kernel.*\\.deb\$";
 @distributions = ('stable', 'testing', 'unstable');
 
 %distmapping = (
-		stable => 'potato',
-		testing => 'woody',
+		stable => 'woody',
+		testing => 'sarge',
 		unstable => 'sid'
 	       );
 
@@ -70,37 +75,14 @@ $distinputcriteria = "^kernel.*\\.deb\$";
 
 @sections = ('main', 'contrib', 'non-free');
 
+@mailtos = ();
+
 @ARGS1 = @ARGV;
 @ARGS2 = ();
 
 $sortpackages = 1;
 $fixstructure = 1;
 
-###############################################################################
-############################# ARGS ############################################
-###############################################################################
-
-while ($_ = shift @ARGS1) {
-  if (/^--/) {
-    if (/^--dl$/ || /^--debug-level$/) {
-      setDebugLevel(shift @ARGS1);
-    }
-    elsif (/^--ql$/ || /^--quit-level$/) {
-      setQuitLevel(shift @ARGS1);
-    }
-    elsif (/^--input$/ || /^--indir$/ || /^--inputdir$/) {
-      $inputdir = shift @ARGS1;
-      $inputdir =~ s/\/$//;
-    }
-    else {
-      push @ARGS2, $_;
-    }
-  }
-  else {
-    push @ARGS2, $_;
-  }
-}
-
 ###############################################################################
 ############################ CONFIG ###########################################
 ###############################################################################
@@ -119,6 +101,49 @@ if (-e $userconfigfile) {
   }
 }
 
+###############################################################################
+############################# ARGS ############################################
+###############################################################################
+
+while ($_ = shift @ARGS1) {
+  if (/^-[a-zA-Z0-9]/) {
+      if (length($_) > 1) {
+	  s/^-//;
+	  foreach $_ (split /(.)/, $_) {
+	      if (length($_) > 0) {
+		  @ARGS1 = ("-$_", @ARGS1);
+	      }
+	  }
+	  $_ = shift @ARGS1;
+      }
+      else {
+	  pdebug(2, "Unknown option $_.");
+      }
+  }
+  if (/^-/) {
+    if (/^--dl$/ || /^--debug-level$/) {
+      setDebugLevel(shift @ARGS1);
+    }
+    elsif (/^--ql$/ || /^--quit-level$/) {
+      setQuitLevel(shift @ARGS1);
+    }
+    elsif (/^-i$/ || /^--input$/ || /^--indir$/ || /^--inputdir$/) {
+      $inputdir = shift @ARGS1;
+      $inputdir =~ s/\/$//;
+    }
+    else {
+      push @ARGS2, $_;
+    }
+  }
+  else {
+    push @ARGS2, $_;
+  }
+}
+
+###############################################################################
+######################### INPUT CONFIG ########################################
+###############################################################################
+
 action(! chdir $inputdir, "Change to dir $inputdir", 2);
 
 if (-e $inputconfigfile) {
@@ -132,7 +157,7 @@ if (-e $inputconfigfile) {
 ############################# HELP ############################################
 ###############################################################################
 
-$version = "0.0.3";
+$version = "0.0.35";
 $versionstring = "Debian package archiver, version $version";
 
 $help =
@@ -143,37 +168,39 @@ options:
   --dl level		 1 = critical, 2 = error, 3 = normal,
 			 4 = message, 5 = debug, 6 = verbose debug (modules).
  --quit-level level	On what level to quit the application, see debug level.
- --version		Prints the version string.
+ -v | --version		Prints the version string.
  --help			Prints this information.
  --copycmd		The install command to use, default $copycmd.
 			 Both packages and .changes files are installed using
 			 this command.
  --movecmd		Command to move files (currently not used at all).
+ --mailcmd		The mail command to use, default $mail.
  --rmcmd		The remove command to use, default $rmcmd.
 			 This can be used to move away the old packages to
 			 some other place.
  --instcmd		DEPRICATED!
- --dest dir		Destination directory. The base directory where all
+ -d | --dest dir       	Destination directory. The base directory where all
   --destdir dir		 the distribution packages reside. Here the
 			 \$distrib/\$major/\$arch/\$section directory structure
 			 will be created. Default $destdir, relative to the
 			 input directory.
- --input dir		This is the directory where the all packages and
-  --indir dir		 corresponding *.changes files that should be
-  --inputdir dir	 installed to the --dest directory,
-			 default $instdir.
+ -i | --input dir      	This is the directory where debarchiver is looking for
+  --indir dir		 new packages corresponding *.changes files that
+  --inputdir dir	 should be installed to the --dest directory.
+			 The default directory is
+			 /var/lib/debarchiver/incoming.
  --lockfile file	The lockfile to use, default $lockfile.
  --cinstall dir		Where the .changes file will be installed to,
 			 empty string to remove it instead, default $cinstall.
  --distinputcritera     The criteria for which binary packages that should be
 			 installed even if it does not have a .changes file,
 			 default $distinputcriteria.
- --addoverride		Automaticly add new packages to the override file.
+ -o | --addoverride    	Automaticly add new packages to the override file.
  --autoscanpackages	Automaticly run dpkg-scanpackages after all new
 			 packages are installed.
  --autoscansources	Automaticly run dpkg-scansources after all new
 			 packages are installed.
- --autoscan		Does both --autoscanpackages and --autoscansources.
+ -a | --autoscan       	Does both --autoscanpackages and --autoscansources.
  --scanall		Scan all distributions, sections, etc.
  --autoscanall		Same as --scanall --autoscan.
  --nosort		Do not sort packages.
@@ -198,6 +225,9 @@ will be parsed. Here you can change the following variables
 			 searched for extra bianry packages that does not need
 			a .changes file to be installed.
 	\$lockfile	The lockfile to use, default $lockfile.
+	@mailtos	The fields in .changes file that should be used for
+			mailing SUCCESS and REJECT messages. If there is an
+			@ char in the arrach it will be used directly.
 ";
 
 ###############################################################################
@@ -205,8 +235,22 @@ will be parsed. Here you can change the following variables
 ###############################################################################
 
 while ($_ = shift @ARGS2) {
-  if (/^--/) {
-    if (/^--version$/) {
+  if (/^-[a-zA-Z0-9]/) {
+      if (length($_) > 1) {
+	  s/^-//;
+	  foreach $_ (split /(.)/, $_) {
+	      if (length($_) > 0) {
+		  @ARGS1 = ("-$_", @ARGS1);
+	      }
+	  }
+	  $_ = shift @ARGS1;
+      }
+      else {
+	  pdebug(2, "Unknown option $_.");
+      }
+  }
+  if (/^-/) {
+    if (/^-v$/ || /^--version$/) {
       print("$versionstring\n");
       exit(0);
     }
@@ -220,10 +264,13 @@ while ($_ = shift @ARGS2) {
     elsif (/^--movecmd$/) {
       $movecmd = shift @ARGS2;
     }
+    elsif (/^--mailcmd$/) {
+      $mailcmd = shift @ARGS2;
+    }
     elsif (/^--rmcmd$/) {
       $rmcmd = shift @ARGS2;
     }
-    elsif (/^--destdir$/ || /^--dest$/) {
+    elsif (/^-d$/ || /^--destdir$/ || /^--dest$/) {
       $destdir = shift @ARGS2;
       $destdir =~ s/\/$//;
     }
@@ -234,7 +281,7 @@ while ($_ = shift @ARGS2) {
       $cinstall = shift @ARGS2;
       $cinstall =~ s/\/$//;
     }
-    elsif (/^--addoverride$/) {
+    elsif (/^-o$/ || /^--addoverride$/) {
       $addoverride = 1;
     }
     elsif (/^--autoscanpackages$/) {
@@ -243,15 +290,18 @@ while ($_ = shift @ARGS2) {
     elsif (/^--autoscansources$/) {
       $autoscansources = 1;
     }
-    elsif (/^--autoscan$/) {
+    elsif (/^-a$/ || /^--autoscan$/) {
       $autoscanpackages = 1;
       $autoscansources = 1;
     }
+    elsif (/^--distinputcriteria/) {
+      $distinputcriteria = shift @ARGS2;
+    }
     elsif (/^--scanall$/) {
-      handleScanAll();
+      $scanall = 1;
     }
     elsif (/^--autoscanall$/) {
-      handleScanAll();
+      $scanall = 1;
       $autoscanpackages = 1;
       $autoscansources = 1;
     }
@@ -286,6 +336,14 @@ action(! chdir $destdir, "Change to dir $destdir", 2);
 
 handleStructureFix();
 
+###############################################################################
+# Fix so that it scan all distributions, sections and so on.
+###############################################################################
+
+if (defined $scanall) {
+  handleScanAll();
+}
+
 ###############################################################################
 # Sort packages.
 ###############################################################################
@@ -326,6 +384,137 @@ sub createPF($$) {
   }
 }
 
+###############################################################################
+######################### EMAIL HANDLING ######################################
+###############################################################################
+
+###############################################################################
+# Name:		determineMailTo
+# Description:	Determine the address to set mail to.
+# Changelog:
+#	2003-02-12	Ola Lundqvist <opal@debian.org>
+#		Wrote it.
+#	2003-02-13	Ola Lundqvist <opal@debian.org>
+#		Extended it with @hostname calculation.
+#	2003-02-28 Ola Lundqvist <opal@debian.org>
+#		Added debugging information.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Switched to using CMeta for ChangeLog meta information.
+#	2003-06-10 Ola Lundqvist <opal@debian.org>
+#		Switched from direct determination of changes owner to use
+#		precalculated data from CMeta{FileOwner}.
+# Uses:		CConf, CMeta hash.
+###############################################################################
+
+sub determineMailTo() {
+    my $to = "";
+    my $toi;
+    foreach $toi (@mailtos) {
+	# Expand to email if there is no email address.
+	if ($toi !~ /\@/) {
+	    $toi = $CConf{$toi};
+	}
+	# Expand to full email address from @hostname field.
+	if ($toi =~ /^\@/) {
+	    $toi = "$CMeta{FileOwner}$toi";
+	    # If still the same...
+	    if ($toi =~ /^\@/) {
+		pdebug(3, "No file owner has been determined, so email address can not be calculated.");
+		$toi = "";
+	    }
+	}
+	if ($toi !~ /^\s*$/) {
+	    if ($to =~ /^\s*$/) {
+		$to = $toi;
+	    }
+	    else {
+		$to = "$to, $toi";
+	    }
+	}
+    }
+    pdebug(5, "Mail will be sent to $to.");
+    return $to;
+}
+
+###############################################################################
+# Name:		email
+# Description:	Send an email to persons, for package with message.
+# Changelog:
+#	2002-09-11 Ola Lundqvist <opal@debian.org>
+#		Stub written.
+#	2003-02-12 Ola Lundqvist <opal@debian.org>
+#		Writen the mail sending function.
+#	2003-02-28 Ola Lundqvist <opal@debian.org>
+#		Added debugging information and fixed arg handling.
+#	2003-03-13 Ola Lundqvist <opal@debian.org>
+#		Debugged the mail command. It stalls.
+# Arguments:	to, package, key, message
+###############################################################################
+sub email($$$$) {
+    my ($to, $package, $key, $message) = @_;
+    if (length($to) > 0) {
+	pdebug(5, "Executing mail command, $mailcmd -s '$package $key' $to.");
+	if (open(M, "|$mailcmd -s '$package $key' '$to'")) {
+	    print M $message;
+	    close(M);
+	}
+	else {
+	    pdebug(2,
+		   "Error executing mail command, $mailcmd -s '$package $key' $to.");
+	}
+	pdebug(5, "Mail exec done.");
+    }
+    else {
+	pdebug(3, "No one to send mail to.");
+    }
+}
+
+###############################################################################
+# Name:		mailSuccess
+#		including the .gz file.
+# Dates:
+#	2002-09-11	Ola Lundqvist <opal@debian.org>
+#		Written.
+#	2003-02-12	Ola Lundqvist <opal@debian.org>
+#		Uses CConf instead of cfile argument.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Switched to using CMeta for ChangeLog meta information.
+# Uses:		%CMeta and maybe %Cfiles in the future.
+###############################################################################
+sub mailSuccess() {
+    # OOPS! We can not read that file after it has been moved!
+    my $message = $CMeta{ChangesContent};
+    pdebug(5, "Mail Success.");
+    email(determineMailTo(),
+	  $CConf{'Source'},
+	  "ACCEPTED",
+	  $message);
+}
+
+###############################################################################
+# Name:		mailReject
+# Dates:
+#	2002-09-12	Ola Lundqvist <opal@debian.org>
+#		Written using data from mailSuccess.
+#	2003-02-12	Ola Lundqvist <opal@debian.org>
+#		Uses CConf instead of cfile argument.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Switched to using CMeta for ChangeLog meta information.
+# Uses:		%CMeta and maybe %Cfiles in the future.
+###############################################################################
+sub mailReject() {
+    # OOPS! We can not read that file after it has been moved!
+    my $message = $CMeta{ChangesContent};
+    if (length($CConf{ERROR}) > 0) {
+	$message += "\n$CConf{ERROR}";
+    }
+    pdebug(5, "Mail Reject.");
+    email(determineMailTo(),
+	  $CConf{'Source'},
+	  "REJECTED",
+          $message);
+}
+
 ###############################################################################
 ########################### HANDLERS ##########################################
 ###############################################################################
@@ -362,6 +551,8 @@ sub handleScanAll() {
 #		2001-07-23	Added lockfile check to distr directory.
 #				Improved documentation.
 #		2002-01-22	Moved lock file checking to its own function.
+#		2003-02-12	Now redirects stderr to stdout for
+#				dpkg-scan* so that it can be logged.
 # Uses:		%dests, $destdir,
 #		$autoscanpackages, $autoscansources.
 # Changes:	$ENV{PWD}
@@ -372,7 +563,7 @@ sub handleScan() {
     $destdir =~ s|/$||;
 
     action(! chdir $destdir, "Change to dir $destdir", 2);
-    destinationLock();
+    &destinationLock();
 
     my $destddir = $destdir;
     $destddir =~ s|^.*/([^/]+)$|$1|;
@@ -387,7 +578,7 @@ sub handleScan() {
       $over =~ s/^(.*)\/[^\/]+$/$1/;
       $over = "$destddir/$over";
       if ($archdest =~ /source$/ && defined $autoscansources) {
-	cmdaction("dpkg-scansources $archdest $over/override.src > $archdest/Sources",
+	cmdaction("dpkg-scansources $archdest $over/override.src 2>&1 > $archdest/Sources",
 		  "Scan source files in $archdest, $over/override.src",
 		  3);
 	cmdaction("gzip $archdest/Sources -c > $archdest/Sources.gz",
@@ -395,7 +586,7 @@ sub handleScan() {
 		  3);
       }
       elsif (defined $autoscanpackages) {
-	cmdaction("dpkg-scanpackages $archdest $over/override > $archdest/Packages.gen",
+	cmdaction("dpkg-scanpackages $archdest $over/override 2>&1 > $archdest/Packages.gen",
 		  "Scan package files in $archdest, $over/override",
 		  3);
 	if ($archdest =~ /binary-all$/) {
@@ -433,6 +624,11 @@ sub handleScan() {
 		      "Concatenate packages files from binary-all and $d.",
 		      3);
 	  }
+	  else {
+	    cmdaction("cat $archdest/Packages.gen > $archdest/Packages",
+                      "Copy packages file from $d.",
+		      3);
+	  }
 	  cmdaction("gzip $archdest/Packages -c > $archdest/Packages.gz",
 		    "Zip it",
 		    3);
@@ -455,7 +651,7 @@ sub handleSorting() {
   if (defined $sortpackages) {
     # First check if a lockfile is created.
       
-    incomingLock();
+    &incomingLock();
 
     # Read the content of this directory.
     opendir(D, ".");
@@ -490,7 +686,7 @@ sub handleSorting() {
       }
     }
 
-    incomingRelease();
+    &incomingRelease();
   }
 }
 
@@ -528,6 +724,16 @@ sub handleStructureFix() {
 	  createPF("$dis/$se/binary-$ar", "Packages");
 	}
 	createPF("$dis/$se/source", "Sources");
+	if (! -e "$dis/$se/override") {
+	  cmdaction("touch $dis/$se/override",
+		    "Create file $dis/$se/override.",
+		    2);
+	}
+	if (! -e "$dis/$se/override") {
+	  cmdaction("touch $dis/$se/override.src",
+		    "Create file $dis/$se/override.src.",
+		    2);
+	}
       }
     }
   }
@@ -556,9 +762,19 @@ sub handleDebOnlyFile($$) {
 ###############################################################################
 # Name:		handleChangesFile
 # Description:	Handles the .changes file.
-# Dates:	2001-06-26	Taken from the main script. Cut and paste with
-#				simple changes.
-# Uses:		parseChanges, pdebug, $copycmd, $rmcmd
+# Changes:
+# 2001-06-26 Ola Lundqvist <opal@debian.org>
+#	Taken from the main script. Cut and paste with simple changes.
+# 2003-03-13 Ola Lundqvist <opal@debian.org>
+#	Added mailSuccess command thing.
+# 2003-06-10 Ola Lundqvist <opal@debian.org>
+#	Now uses uploaderIsChangesFileOwner to make sure that the owner can be
+#	calculated before the changes file will be moved or something similar.
+#       Also added rejectChangesFile to vrfycmd so that messages will be sent
+#	properly if it is rejected.
+#       Moved parseChanges before sig verify and uploaderIsChangesFileOwner to
+#	make sure that CMeta is created before that.
+# Uses:		parseChanges, pdebug, $copycmd, $rmcmd, uploaderIsChangesFileOwner
 # Changes:	See parseChanges.
 # Arguments:	The .changes file.
 # Returns:	nothing
@@ -568,11 +784,81 @@ sub handleChangesFile($) {
   my ($cfile) = @_;
 
   parseChanges($cfile);
+  uploaderIsChangesFileOwner($cfile);
+  if ($verifysignatures) {
+    if (system($vrfycmd,$cfile)) { # non-zero == verification failure
+      rejectChangesFile();
+      pdebug(2, "Signature verification failed for $cfile");
+      return;
+    }
+  }
+
   my $file;
   foreach $file (keys %CFiles) {
     handlePackageFile($file);
   }
   installChangesFile($cfile);
+  mailSuccess();
+}
+
+###############################################################################
+# Name:		uploaderIsChangesFileOwner
+# Description:	Sets CMeta{FileOwner} from 
+# Changes:	CMeta
+# Dates:
+#	2003-06-10	Ola Lundqvist <opal@debian.org
+#		Wrote it.
+###############################################################################
+
+sub uploaderIsChangesFileOwner($) {
+  my ($cfile) = @_;
+  my @temp = stat($cfile);
+  if (@temp) {
+      my $tmp = $temp[4];
+      $CMeta{FileOwner} = getpwuid($tmp);
+      pdebug(5, "$cfile is owned by $CMeta{FileOwner}");
+  }
+  else {
+      pdebug(3, "Can not stat file $cfile, so unable to calculate email address.");
+      $CMeta{FileOwner} = "";
+  }
+}
+
+###############################################################################
+# Name:		rejectChangesFile
+# Description:	Reject the changes file so that it is moved away.
+# Uses:		CFiles, CConf
+# Dates:
+#	2003-02-12	Ola Lundqvist <opal@debian.org
+#		Wrote it.
+#	2003-02-28	Ola Lundqvist <opal@debian.org
+#		Implemented the commands in the foreach loop.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Switched to using CMeta for ChangeLog meta information.
+###############################################################################
+
+sub rejectChangesFile() {
+    my $file;
+    if (! -d "REJECT") {
+	action(! mkpath ("REJECT", 0, 0775),
+	       "Making REJECT directory.",
+	       2);
+    }
+    if ($CMeta{ChangesFile} != "") {
+	cmdaction("$mvcmd $CMeta{ChangesFile} REJECT/",
+		  "Move .changes to REJECT dir.",
+		  2);
+    }
+    foreach $file (keys %CFiles) {
+	if (-f $file) {
+	    cmdaction ("$mvcmd $file REJECT/",
+		       "Move $file to REJECT dir.",
+		       3);
+	}
+	else {
+	    $CConf{ERROR} = "$CConf{ERROR}File $file can not be moved because it is already installed.\n";
+	}
+    }
 }
 
 ###############################################################################
@@ -587,7 +873,7 @@ sub handleChangesFile($) {
 ###############################################################################
 
 sub handlePackageFile ($) {
-  my ($file) = @_;
+  my ($file, $action) = @_;
   my $distrd = $CConf{Distribution};
 
   pdebug(5, "File $_:  $CFiles{$file}");
@@ -659,7 +945,11 @@ sub handlePackageFile ($) {
 ###############################################################################
 # Name:		installChangesFile
 # Description:	Moves the changes file to the right place, or remove it.
-# Dates:	2001-06-10	
+# Changes:
+#	2001-06-10 Ola Lundqvist <opal@debian.org>
+#		Wrote it.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Now CConf{ChangesFile} is changed when copying it.
 # Arguments:	The .changes file.
 #		Where to place it.
 # Returns:	($major, $section)
@@ -708,6 +998,13 @@ sub incomingRelease() {
 sub incomingError() {
     &setErrorHandler(undef);
     &removeLockfile("$lockfile");    
+    &setErrorHandler(\&rejectError);    
+    &rejectChangesFile();
+}
+
+sub rejectError() {
+    &setErrorHandler(undef);
+    &mailReject();
 }
 
 sub destinationLock() {
@@ -736,6 +1033,7 @@ sub destinationError() {
 # Arguments:   	$lockfile
 # Returns:	nothing
 ###############################################################################
+
 sub createLockExit($) {
     my ($lockfile) = @_;
     if (-e $lockfile) {
@@ -754,6 +1052,7 @@ sub createLockExit($) {
 # Needs:	$rmcmd $lockfile
 # Returns:	nothing
 ###############################################################################
+
 sub removeLockExit() {
     my ($lockfile) = @_;
     removeLockfile($lockfile);
@@ -767,6 +1066,7 @@ sub removeLockExit() {
 # Arguments:    $lockfile
 # Returns:	nothing
 ###############################################################################
+
 sub removeLockfile() {
     my ($lockfile) = @_;
     cmdaction("rm $lockfile",
@@ -842,7 +1142,11 @@ sub parseFileName($) {
 # Name:		parseDebOnlyFile
 # Description:	Parses a debian deb file and extracs the information in the
 #		way that parseChanges does.
-# Dates:	2001-06-29	Written with info from parseChanges.
+# Dates:
+#	2001-06-29 Ola Lundqvist <opal@debian.org>
+#		Written with info from parseChanges.
+#	2003-02-12 Ola Lundqvist <opal@debian.org>
+#		Added ChangesFile to CConf hash.
 # Arguments:	A deb file name.
 # Changes:	see parseChanges
 ###############################################################################
@@ -856,6 +1160,7 @@ sub parseDebOnlyFile($$) {
   my $size;
   my $desc;
   %CConf = (Distribution => $distr);
+  %CMeta = (ChangesFile => "", ChangesContent => "");
   %CFiles = ();
   %CDesc = ();
   my @cmdres = readcommand("dpkg-deb -f $kfile");
@@ -907,8 +1212,17 @@ sub parseDebOnlyFile($$) {
 ###############################################################################
 # Name:		parseChanges
 # Description:	Parses a debian changelog file and extracs the information.
-# Dates:	2001-06-10	Written.
-#		2001-06-26	Changed print to pdebug.
+# ChangeLog:
+#	2001-06-10 Ola Lundqvist <opal@debian.org>
+#		Written.
+#	2001-06-26 Ola Lundqvist <opal@debian.org>
+#		Changed print to pdebug.
+#	2002-09-11 Ola Lundqvist <opal@debian.org>
+#		Added uploaders field to the description.
+#	2003-02-12 Ola Lundqvist <opal@debian.org>
+#		Added ChangesFile to CConf hash.
+#	2003-03-14 Ola Lundqvist <opal@debian.org>
+#		Switched to using CMeta for ChangeLog meta information.
 # Arguments:	.changes file name.
 # Changes:	
 #	%CConf
@@ -920,6 +1234,11 @@ sub parseDebOnlyFile($$) {
 #		'Distribution' => The intended distribution.
 #		'Urgency' => How urgent the package installation is.
 #		'Maintainer' => The package maintainer.
+#		'Uploaders' => The other package maintainers.
+#		... => other undocumentated things that can be used.
+#	%CMeta
+#		'ChangesFile' => The file.
+#		'ChangesContent' => The content of the ChangeLog file.
 #	%CFiles
 #		$filename => "$hash $size $section $type"
 #	%CDesc
@@ -928,13 +1247,18 @@ sub parseDebOnlyFile($$) {
 
 sub parseChanges($) {
   my ($file) = @_;
+  if ($file =~ /\|$/) {
+      pdebug(2, "The changes file is not allowed to end in |, because that can cause a failure\nin the debarchiver program.");
+  }
   open (F, $file);
   my $state = "";
   my $line;
   %CConf = ();
+  %CMeta = (ChangesFile => $file, ChangesContent => "");
   %CFiles = ();
   %CDesc = ();
   while ($line = <F>) {
+    $CMeta{ChangesContent} = $CMeta{ChangesContent} . $line;
     # The state to just put the line in the hash.
     if ($line =~ /^\s*$/) {
       next;
@@ -1090,7 +1414,7 @@ debarchiver [options]
   --dl level		 1 = critical, 2 = error, 3 = normal,
 			 4 = message, 5 = debug, 6 = verbose debug (modules).
  --quit-level level	On what level to quit the application, see debug level.
- --version		Prints the version string.
+ -v | --version		Prints the version string.
  --help			Prints this information.
  --copycmd		The install command to use, default $copycmd.
 			 Both packages and .changes files are installed using
@@ -1100,12 +1424,12 @@ debarchiver [options]
 			 This can be used to move away the old packages to
 			 some other place.
  --instcmd		DEPRICATED!
- --dest dir		Destination directory. The base directory where all
+ -d | --dest dir       	Destination directory. The base directory where all
   --destdir dir		 the distribution packages reside. Here the
 			 \$distrib/\$major/\$arch/\$section directory structure
 			 will be created. Default $destdir, relative to the
 			 input directory.
- --input dir		This is the directory where the all packages and
+ -i | --input dir	This is the directory where the all packages and
   --indir dir		 corresponding *.changes files that should be
   --inputdir dir	 installed to the --dest directory,
 			 default $instdir.
@@ -1115,12 +1439,12 @@ debarchiver [options]
  --distinputcritera     The criteria for which binary packages that should be
 			 installed even if it does not have a .changes file,
 			 default $distinputcriteria.
- --addoverride		Automaticly add new packages to the override file.
+ -o | --addoverride    	Automaticly add new packages to the override file.
  --autoscanpackages	Automaticly run dpkg-scanpackages after all new
 			 packages are installed.
  --autoscansources	Automaticly run dpkg-scansources after all new
 			 packages are installed.
- --autoscan		Does both --autoscanpackages and --autoscansources.
+ -a | --autoscan       	Does both --autoscanpackages and --autoscansources.
  --scanall		Scan all distributions, sections, etc.
  --autoscanall		Same as --scanall --autoscan.
  --nosort		Do not sort packages.
@@ -1148,7 +1472,14 @@ will be parsed. Here you can change the following variables:
 			 searched for extra bianry packages that does not need
 			a .changes file to be installed.
 	$lockfile	The lockfile to use, default $lockfile.
-
+	@mailtos	An array of strings that should be mailed to.
+			If the string contains en email address that one is
+			used. If it contains an incomplete email address, i.e.
+    			@hostname, the username owning the file is used @
+    			the hostname specified. If no @ character is found in
+    			the string, it is considered as a field in the
+    			.changes file. Such a field can for example be
+    			Maintainer or Uploaders.
 
 =head1 AUTHOR
 
